//////////////////////////////////////////////////////////////////////////////
//
//	Skeleton.c 
// 
//  WDM  API
//
//////////////////////////////////////////////////////////////////////////////
 #define DRIVER
 
#include <ntddk.h>
#include <stdio.h>
#include "calcdrv.h"

NTSTATUS
DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath)
{
	NTSTATUS			ntStatus = STATUS_SUCCESS;
	ANSI_STRING			devName, linkName;
	UNICODE_STRING		uniDevName, uniLinkName;	
	CHAR				DeviceNameBuffer[NAME_MAX];
	CHAR				DeviceLinkBuffer[NAME_MAX];
	PDEVICE_OBJECT		pDeviceObject = NULL;
	PDEVICE_EXTENSION	deviceExtension;

	strcpy(DeviceNameBuffer, "\\Device\\");
	strcpy(DeviceLinkBuffer, "\\DosDevices\\");
	strcat(DeviceNameBuffer, DRIVER_NAME);
	strcat(DeviceLinkBuffer, DRIVER_NAME);

	RtlInitAnsiString(&devName, DeviceNameBuffer);
	RtlInitAnsiString(&linkName, DeviceLinkBuffer);
	RtlAnsiStringToUnicodeString(&uniDevName, &devName, TRUE);
	RtlAnsiStringToUnicodeString(&uniLinkName, &linkName, TRUE);

	ntStatus = IoCreateDevice(DriverObject, 
							  sizeof(DEVICE_EXTENSION),
							  &uniDevName,
							  FILE_DEVICE_UNKNOWN,
							  0,
							  FALSE,
							  &pDeviceObject);

	IoCreateUnprotectedSymbolicLink(&uniLinkName, &uniDevName);

	RtlFreeUnicodeString(&uniDevName);
	RtlFreeUnicodeString(&uniLinkName);


	DriverObject->MajorFunction[IRP_MJ_CREATE]	= Dispatch_Create;
	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = Skeleton_Dispatch;
	DriverObject->MajorFunction[IRP_MJ_CLOSE]	= Dispatch_Close;
	DriverObject->MajorFunction[IRP_MJ_READ]	= Dispatch_Read;

	DbgPrint("DriverEntry %s %s",&uniDevName,&uniLinkName);

	return ntStatus;
}


NTSTATUS
Skeleton_Dispatch(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp)
{
	NTSTATUS			ntStatus = STATUS_SUCCESS;
	PIO_STACK_LOCATION	irpStack;
	PDEVICE_EXTENSION	deviceExtension;
	PVOID				ioBuffer;
	ULONG				ioCtrlCode, outLen, inLen;
	
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;

	irpStack = IoGetCurrentIrpStackLocation(Irp);
	deviceExtension =	DeviceObject->DeviceExtension;

	ioBuffer	= Irp->AssociatedIrp.SystemBuffer;
	outLen		= irpStack->Parameters.DeviceIoControl.OutputBufferLength;
	inLen		= irpStack->Parameters.DeviceIoControl.InputBufferLength;

	DbgPrint("Skeleton_Dispatch %d",&inLen);

	switch(irpStack->MajorFunction)
	{
		case IRP_MJ_DEVICE_CONTROL:
			ioCtrlCode = irpStack->Parameters.DeviceIoControl.IoControlCode;

			switch(ioCtrlCode)
			{
				case SUM_TWO_INTEGER:
					
					*((PLONG) ioBuffer) = 
							((PLONG) ioBuffer)[0] + ((PLONG) ioBuffer)[1];
					 
					Irp->IoStatus.Information = sizeof(LONG);
					break;

				default:
					break;
			}

		default:
			break;
	}
	
	IoCompleteRequest(Irp, IO_NO_INCREMENT);
	
	return ntStatus;
}




NTSTATUS
Dispatch_Create(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp)
{
	NTSTATUS			ntStatus = STATUS_SUCCESS;
	PIO_STACK_LOCATION	irpStack;

	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;

	IoCompleteRequest(Irp, IO_NO_INCREMENT);

	DbgPrint("Dispatch_Create");

	return ntStatus;
}

NTSTATUS
Dispatch_Close(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp)
{
	NTSTATUS			ntStatus = STATUS_SUCCESS;
	PIO_STACK_LOCATION	irpStack;

	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;

	IoCompleteRequest(Irp, IO_NO_INCREMENT);

	DbgPrint("Dispatch_Close");
	
	return ntStatus;
}

NTSTATUS
Dispatch_Read(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp)
{
	NTSTATUS			ntStatus = STATUS_SUCCESS;
	PIO_STACK_LOCATION	irpStack;

	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;

	IoCompleteRequest(Irp, IO_NO_INCREMENT);

	DbgPrint("Dispatch_Read");

	return ntStatus;
}
//////////////////////////////////////////////////////////////////////////////
//
//       Source End
//
//////////////////////////////////////////////////////////////////////////////